import os

from . import settings


def actual_path(base_path, file_type, path):

    # make sure the path doesn't begin with a slash
    # (this screws up os.path.join)
    if len(path) > 0 and path[0] == '/':
        path = path[1:]

    actual = os.path.abspath(os.path.join(base_path, file_type, path))

    # make sure we end with a slash
    if actual[-1] != '/':
        actual = actual + '/'

    return actual


def actual_url(base_url, file_type, path, filename=None):
    """Return the full URL of the resource; this is the base_url
    (usually settings.FCK_CONNECTOR_URL), with a subsequent path of
    /file_type/path."""

    # we need to remain sane in the face of slashes -- this is especially
    # true when the media type is mapped to root, and we're viewing the
    # root.

    # so the base_url should *not* end in a slash,
    # the file_type should *both* begin and end with a slash,
    # and the path should *not* begin with a slash

    if base_url[-1] == '/':
        base_url = base_url[:-1]

    if len(file_type) == 0:

        file_type = '/'

    if file_type[0] != '/':

        file_type = '/%s' % file_type

    if file_type[-1] != '/':

        file_type = '%s/' % file_type

    if len(path) > 0 and path[0] == '/':
        path = path[1:]

    folder_path = "%s%s%s" % (base_url, file_type, path)

    if filename is not None:
        # append the filename
        if folder_path[-1] != '/':
            folder_path = "%s/%s" % (folder_path, filename)
        else:
            folder_path = "%s%s" % (folder_path, filename)

    return folder_path


def get_resource_type_folder(request):
    """Map a resource_type passed in from the Request to the real
    sub-folder using the RESOURCE_TYPE_MAP.  Returns a tuple containing
    the specified resource type and the mapped path.  If a mapping
    does not exist, the mapped path defaults to the specified
    resource type."""

    resource_type = request.REQUEST.get('Type', None)
    resource_type_path = settings.RESOURCE_TYPE_MAP.get(resource_type,
                                                        resource_type)

    return (resource_type, resource_type_path)



from django.conf.urls import patterns, include, url

from django.conf import settings
from django.conf.urls.static import static

urlpatterns = static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT) + patterns(
    '',
    url(r'^grappelli/', include('grappelli.urls')),
    url(r'', include('django_pages.urls')),
    ) 



# -*- encoding: utf-8 -*-

from django.db import models

from ..language.models import Language


class MenuItem(models.Model):
    '''
    Stores menuitem information.
    It is recommended to write urls in SEO format, e.g.: this_is_my_site_describing_something.
    Is related to :model: `django_pages.Menu`.
    '''

    lang = models.ForeignKey(Language)
    menuitem_name = models.CharField('MenuItem name', max_length=200)
    url = models.CharField('Url', max_length=200, unique=True)
    position = models.IntegerField('Position', blank=True)

    def __unicode__(self):

        return '%s - %s' % (self.menuitem_name, self.lang)

    def save(self, *args, **kwargs):
        """
        if MenuItem does not contain position, set it to last_item's_position + 1
        """

        if not self.position:

            last_position = self.get_last_position()

            self.position = last_position + 1

        super(MenuItem, self).save(*args, **kwargs)

    def get_last_position(self):
        """
        returns last_item's_position or 0 (no items)
        """

        other_objects = MenuItem.objects.filter(lang=self.lang).order_by('-position')

        if other_objects.count():

            last_position = other_objects[0].position

        else:

            last_position = 0

        return last_position

    def is_first(self):
        """
        returns True if self.position is 1, False otherwise
        """

        if self.position == 1:

            return True

        return False

    def is_last(self):
        """
        if self.position is last, return True, False otherwise

        @return bool
        """

        last_position = self.get_last_position()

        if self.position == last_position:

            return True

        return False

    def increase_position(self):
        """
        moves this item to self.position + 1
        """

        if self.is_last():

            pass

        else:

            self.swap_with(self.position + 1)

    def decrease_position(self):
        """
        moves this item to self.position - 1
        """

        if self.is_first():

            pass

        else:

            self.swap_with(self.position - 1)

    def swap_with(self, position):
        """
        handles item moving, makes sure there are no items with same position
        """

        object_to_swap_with = MenuItem.objects.get(lang=self.lang, position=position)

        object_to_swap_with.position = self.position
        object_to_swap_with.save()

        self.position = position
        self.save()

    def is_current(self, page_url):
        """
        checks if this is currently active menuitem

        @return bool
        """

        if page_url == self.url or (self.page.index and self.page.active and page_url is None):

            return True

        return False



# -*- encoding: utf-8 -*-

from django import forms

import reversion

from ..common.admin_actions import activate, deactivate
from .models import Language


class LanguageAdminForm(forms.ModelForm):

    country_code = forms.RegexField(r'^[A-z]{2,3}$', error_messages={'invalid': '2-3 letter combination required'})

    class Meta:

        model = Language


class LanguageAdmin(reversion.VersionAdmin):

    form = LanguageAdminForm
    fields = (('language', 'country_code'), 'flag', 'default', 'active')
    list_display = ('language', 'country_code', 'default', 'active')
    actions = [activate, deactivate]



# -*- encoding: utf-8 -*-

from django.http import Http404

from ..common.errors import ConfigurationError
from .models import Language


def get_language(url_data):
    '''
    checks for language in data from url parsing

    @param url_data: dict
    @return Language object
    '''

    if not url_data['country_code']:

        language = Language.objects.get(default=True)

        if not language.active:

            raise ConfigurationError('There is no default language active, please activate it in admin')

    else:

        try:

            language = Language.objects.get(country_code=url_data['country_code'])

        except Language.DoesNotExist:

            raise Http404

    return language


def get_languages():
    '''
    returns Language QuerySet or ()
    '''

    languages = list(Language.objects.filter(active=True))

    if len(languages) > 1:

        return languages

    return tuple()



# -*- encoding: utf-8 -*-

from django.contrib import admin

class CommentAdmin(admin.ModelAdmin):

    list_display = ('id', '__unicode__', 'user', 'post', 'ip')
    list_display_links = ('id', '__unicode__')



# -*- encoding: utf-8 -*-

from django import forms


class CommentForm(forms.Form):

    user = forms.CharField(max_length=100, widget=forms.TextInput(attrs={'class': 'input-xlarge'}))
    comment = forms.CharField(max_length=1000, widget=forms.Textarea(attrs={'class': 'input-xlarge'}))

    one = forms.BooleanField(required=False)
    two = forms.BooleanField(required=False)
    three = forms.BooleanField(required=False)
    four = forms.BooleanField(required=False)



# -*- encoding: utf-8 -*-

import re

from django.http import Http404, HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.text import slugify
from django.views.decorators.cache import cache_page

from .comments import handle_comment, set_humanity_check, translate_humanity
from .comments.forms import CommentForm
from .language import get_language, get_languages
from .language.models import Language
from .looks import get_template
from .metadata import get_metadata
from .pages import get_page, get_index_page, get_post, get_paginated_posts
from .pages.models import Page
from .site import get_site


def parse_url(url):
    '''
    parses url format is:
        language/page~pagenum/~post

        example:
            en/my-page-with-something-interresting~5/~post-on-page-5

    @param url: string
    @return dict
    '''

    urlpattern = re.compile(r'''
                          ^                                         # beginning of string
                          ((?P<country_code>[A-z]{2,3})/){0,1}      # country_code match - any 2 or 3 chars, zero or one times
                          (?P<page>[A-z0-9-._]{4,}){0,1}            # page url
                          (~(?P<page_num>\d*)){0,1}                 # ~page number
                          (/~(?P<post>[A-z0-9-._]*)){0,1}           # /~post title
                          $                                         # end of string
                          ''', re.VERBOSE)

    urlmatch = re.match(urlpattern, url)

    if not urlmatch:

        raise Http404

    return urlmatch.groupdict()


@cache_page(10)
def main_view(request, url):

    url_result = parse_url(url)

    current_site = get_site()
    current_template = get_template()  # sets tuple (template_name, posts_on_page)

    language = get_language(url_result)

    if not url_result['page']:

        page = get_index_page(language)

    else:

        page = get_page(url_result['page'], language)

    menuitems = page.link.lang.menuitem_set.select_related('page').filter(page__active=True).order_by('position').all()

    for menuitem in menuitems:

        menuitem.current = menuitem.is_current(url_result['page'])

    meta_data = get_metadata(page)

    page_num = url_result['page_num'] or 1

    if url_result['post']:

        posts = get_post(page, url_result['post'])

        template_page = 'post.html'

        if request.method == 'POST':

            form = handle_comment(request, posts)

            set_humanity_check(request)
            form.humanity = translate_humanity(request)

        else:

            form = CommentForm()

            set_humanity_check(request)
            form.humanity = translate_humanity(request)

    else:

        posts = get_paginated_posts(page, page_num, current_template[1])

        template_page = 'page.html'

    site_content = {'site': current_site,
                    'languages': get_languages(),
                    'current_language': language,
                    'menuitems': menuitems,
                    'page': page,
                    'metadata': meta_data,
                    'posts': posts, }

    try:

        site_content['form'] = form

    except NameError:

        pass

    template = '%s/%s' % (current_template[0], template_page)

    return render_to_response(template, {'site_content': site_content}, context_instance=RequestContext(request))


@cache_page(60 * 60 * 24)
def robots(request):
    """
    generates robots.txt, which pretty much does not change
    """

    site = get_site()
    domain = site.domain

    data = '''Sitemap: http://%s/sitemap.xml
User-agent: *
Disallow: /admin/
Disallow: /media/
Disallow: /static/
''' % domain

    return HttpResponse(data, content_type='text/plain')


@cache_page(60 * 60 * 24)
def generate_sitemap(request):
    """
    generates /sitemap.xml
    """

    data = []

    site_url = 'http://%s/' % get_site().domain

    for language in Language.objects.all():

        for page in Page.objects.filter(link__lang=language):

            page_url = '%s%s/%s' % (site_url, language.country_code, page.link.url)

            data.append(page_url)

            for post in page.post_set.all():

                data.append('%s/~%s' % (page_url, slugify(post.title)))

    return render_to_response('sitemap.xml', {'urls': data}, context_instance=RequestContext(request), mimetype='application/xml')



import os.path
import urlparse
from django.conf import settings

# See __init__.py for documentation on setting the values of
# FCKEDITOR_CONNECTOR_ROOT and FCKEDITOR_CONNECTOR_URL

# FCKEDITOR_CONNECTOR_PREFIX defines the path prefix inserted between
# the MEDIA_ROOT and item type (ie, 'Image') when forming the actual path
#
# For example, if MEDIA_ROOT is set to '/var/www', this setting:
#
# FCKEDITOR_CONNECTOR_PREFIX = 'Media'
#
# would cause FCKeditor Connector to look in '/var/www/Media/Image' for
# images when browsing the server.

FCKEDITOR_CONNECTOR_PREFIX = 'uploaded'


FCKEDITOR_CONNECTOR_ROOT = os.path.join(settings.MEDIA_ROOT,
                                        FCKEDITOR_CONNECTOR_PREFIX)
FCKEDITOR_CONNECTOR_URL = urlparse.urljoin(settings.MEDIA_URL,
                                        FCKEDITOR_CONNECTOR_PREFIX)


# RESOURCE_TYPE_MAP allows you to map FCKeditor's resource types
# (Image, File, etc) to other paths; these are still appended to
# both [FCKEDITOR_CONNECTOR_ROOT|FCKEDITOR_CONNECTOR_URL] + PREFIX

RESOURCE_TYPE_MAP = {
    'Image' : 'images',
    'Flash' : 'flash',
    'Media' : 'media',
    'File' : '',
    }

IGNORE_FOLDERS = ['.svn',]



# -*- encoding: utf-8 -*-

from django.db import models


class Template(models.Model):
    '''
    Stores template information.
    Template name is used for template selection in templates foled of your project
    '''

    template = models.CharField('Template name', max_length=200)
    posts_per_page = models.PositiveIntegerField('Posts per page', default=10)
    active = models.BooleanField('Active', default=False, blank=True)

    def __unicode__(self):

        return self.template



# -*- encoding: utf-8 -*-

"""
This file just imports admins from all packages so Django finds them
"""

from django.contrib import admin

from .comments.models import Comment
from .feed.models import FeedSettings
from .language.models import Language
from .looks.models import Template
from .menu.models import MenuItem
from .metadata.models import MetaSet, MetaData
from .pages.models import Page, Post
from .site.models import Site

from .comments.admin import CommentAdmin
from .feed.admin import FeedAdmin
from .language.admin import LanguageAdmin
from .looks.admin import LooksAdmin
from .menu.admin import MenuItemAdmin
from .metadata.admin import MetaSetAdmin, MetaDataAdmin
from .pages.admin import PageAdmin, PostAdmin
from .site.admin import SiteAdmin

admin.site.register(Comment, CommentAdmin)
admin.site.register(FeedSettings, FeedAdmin)
admin.site.register(Language, LanguageAdmin)
admin.site.register(Template, LooksAdmin)
admin.site.register(MenuItem, MenuItemAdmin)
admin.site.register(MetaSet, MetaSetAdmin)
admin.site.register(MetaData, MetaDataAdmin)
admin.site.register(Page, PageAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(Site, SiteAdmin)



# -*- encoding: utf-8 -*-

from django.db import models
from django.core.files.storage import FileSystemStorage

from ..settings import FLAG_UPLOAD_DIR

IMAGES_DIR = FileSystemStorage(FLAG_UPLOAD_DIR)


class Language(models.Model):
    '''
    Stores single language with optional flag image.
    Country Code will be part of URL in lowercase.
    '''

    language = models.CharField('Language', max_length=150, unique=True)
    country_code = models.CharField('Country code', max_length=3, unique=True)
    flag = models.ImageField(storage=IMAGES_DIR, upload_to='languages', blank=True)
    default = models.BooleanField('Display as default language?', blank=True)
    active = models.BooleanField('Active', blank=True, default=True)

    def __unicode__(self):

        return self.language

    def save(self, *args, **kwargs):
        '''
        override save method to check wheter it is first language.
        and if so, check default to True.
        if not, check if some other language has default set to True.
        if not, do not allow to set it to False.
        '''

        if not self.default:

            if not Language.objects.exists():

                self.default = True

        else:

            other_default = Language.objects.filter(default=True).exclude(id=self.id)

            if other_default:

                other_default = other_default[0]
                other_default.default = False
                # call super.save on this, so we don't check all this again
                super(Language, other_default).save(*args, **kwargs)

            self.default = True

        self.country_code = self.country_code.lower()

        super(Language, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        '''
        override delete method to check if any other language has default set.
        if not, set language with lowest ID as default.
        '''

        super(Language, self).delete(*args, **kwargs)

        if not Language.objects.filter(default=True):

            new_default = Language.objects.all().order_by('id')

            if new_default:

                new_default = new_default[0]
                new_default.default = True
                new_default.save()



# -*- encoding: utf-8 -*-

from django.conf import settings


try:
    ADMIN_MEDIA_PREFIX = settings.ADMIN_MEDIA_PREFIX

except AttributeError:
    ADMIN_MEDIA_PREFIX = '/static/admin/'

# where to upload flag images /media/FLAG_UPLOAD_DIR
try:
    FLAG_UPLOAD_DIR = settings.FLAG_UPLOAD_DIR

except AttributeError:
    FLAG_UPLOAD_DIR = settings.MEDIA_ROOT



# -*- encoding: utf-8 -*-

from django.db import models

from ..language.models import Language


class MetaSet(models.Model):
    '''
    Stores MetaData configurations
    '''

    language = models.ForeignKey(Language)
    name = models.CharField('MetaSet name', max_length=200)

    def __unicode__(self):

        return self.name


class MetaData(models.Model):
    '''
    Stores MetaData item
    '''

    meta_set = models.ForeignKey(MetaSet)
    name = models.CharField('name', max_length=200)
    content = models.TextField('content')

    class Meta:

        verbose_name_plural = 'Meta data'

    def __unicode__(self):

        return '%s@%s' % (self.name, self.meta_set)





# -*- encoding: utf-8 -*-

"""
Custom errors definition
"""

from django.core.exceptions import ImproperlyConfigured


class ConfigurationError(ImproperlyConfigured):

    def __init__(self, *args, **kwargs):

        super(ConfigurationError, self).__init__(*args, **kwargs)



# -*- encoding: utf-8 -*-

import random

from django.utils.html import strip_tags

from .forms import CommentForm
from .models import Comment


def get_client_ip(request):
    """
    Returns requester's IP address from HTTP request

    @param request: Http Request
    @return string
    """

    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')

    if x_forwarded_for:

        ip = x_forwarded_for.split(',')[0]

    else:

        ip = request.META.get('REMOTE_ADDR')

    return ip


def set_humanity_check(request):
    """
    sets session['humanity'] to dictionary {0: True/False, 1:True/False, .. 3:True/False}

    @param request: Http Request
    @return None
    """

    result = {}

    for i in xrange(4):

        if random.randint(0, 1):

            result[i] = True

        else:

            result[i] = False

    request.session['humanity'] = result


def translate_humanity(request):
    """
    translates request.session['humanity'] dictionary {0:True, 1:False, ..}
    into 'One, Two, Three' according to numbers that are True

    @param request: Http Request
    @return string
    """

    text = []

    for i in xrange(4):

        if request.session['humanity'][i]:

            if i == 0:
                text.append('one')

            elif i == 1:
                text.append('two')

            elif i == 2:
                text.append('three')

            elif i == 3:
                text.append('four')

        elif request.session['humanity'][i] == {0: False, 1: False, 2: False, 3: False}:

            text.append('None')

    return ', '.join(text)


def is_human(request, data):

    if request.session['humanity'] == data:

        return True

    return False


def handle_comment(request, post):
    '''
    handles comment and either saves it or displays error

    @param request: Http Request
    @param post: Post object
    @return None or errorfrom django.utils.timezone import make_aware, get_current_timezone
    '''

    form = CommentForm(request.POST)
    ip = get_client_ip(request)

    if form.is_valid():

        user = strip_tags(form.cleaned_data['user'])
        comment = strip_tags(form.cleaned_data['comment'])

        one = form.cleaned_data['one']
        two = form.cleaned_data['two']
        three = form.cleaned_data['three']
        four = form.cleaned_data['four']

        if is_human(request, {0: one, 1: two, 2: three, 3: four}):

            if Comment.objects.filter(post=post, user=user, comment=comment):

                return CommentForm()

            comment = Comment(post=post, user=user, comment=comment, ip=ip)
            comment.save()

            return CommentForm()

        else:

            form.spam = 'wrong antispam check'
            return form

    else:

        return form



# -*- encoding: utf-8 -*-

from distutils.core import setup
from setuptools import find_packages

setup(
    name='django-pages',
    version='1.1.1',
    author=u'Lukas Nemec',
    author_email='lu.nemec@gmail.com',
    url='https://github.com/lunemec/django-pages',
    license='see LICENCE.txt',
    description='Simple CMS for django',
    long_description=open('README.md').read(),
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(),
)



# -*- encoding: utf-8 -*-

from django.db import models

from ..pages.models import Post


class Comment(models.Model):
    '''
    Stores comment data.
    Is related to :model: `pages.Post`.
    '''

    post = models.ForeignKey(Post)
    user = models.CharField('User', max_length=100)
    comment = models.TextField('Comment', max_length=1000)
    ip = models.CharField('IP', max_length=100, blank=True)

    def __unicode__(self):

        return '%s@%s' % (self.user, self.comment[:50])



## -*- encoding: utf-8 -*-
#
#from datetime import datetime
#
#from django.http import Http404
#from django.test import TestCase
#from django.utils.timezone import make_aware, get_current_timezone
#
#
#class ModelSiteTest(TestCase):
#
#    def setUp(self):
#
#        self.site = Site(domain='lnemec.tk', display_name='Lnemecdomain', tagline='shitt', footer='as')
#        self.site.save()
#
#    def test_unicode(self):
#
#        self.assertEqual(self.site.__unicode__(), self.site.domain)
#
#
#class ModelLanguageTest(TestCase):
#
#    def setUp(self):
#
#        self.lang = Language()
#        self.lang.language = 'English'
#        self.lang.country_code = 'EN'
#        self.lang.save()
#
#        self.lang_from_db = Language.objects.get(pk=1)
#
#        self.lang2 = Language()
#        self.lang2.language = 'Czech'
#        self.lang2.country_code = 'CZ'
#        self.lang2.save()
#
#    def test_unicode(self):
#
#        self.assertEqual(self.lang.__unicode__(), self.lang.language)
#
#    def test_country_code_smallcase_in_db(self):
#
#        self.assertEqual(self.lang_from_db.country_code, 'en')
#
#    def test_country_code_smallcase_after_save(self):
#
#        self.assertEqual(self.lang.country_code, 'en')
#
#    def test_first_lang_default_true(self):
#
#        self.assertTrue(self.lang_from_db.default)
#
#    def test_second_lang_default_false(self):
#
#        self.assertFalse(self.lang2.default)
#
#    def test_default_swap(self):
#
#        self.lang2.default = True
#        self.lang2.save()
#
#        self.lang = Language.objects.get(pk=1)
#
#        self.assertTrue(self.lang2.default)
#        self.assertFalse(self.lang.default)
#
#    def test_default_swap_after_delete(self):
#
#        self.lang2.delete()
#
#        self.lang = Language.objects.get(pk=1)
#
#        self.assertTrue(self.lang.default)
#
#    def test_can_be_deleted(self):
#
#        self.lang.delete()
#
#    def test_languages_empty(self):
#
#        self.lang.delete()
#        self.lang2.delete()
#
#        lang_count = Language.objects.all().count()
#
#        self.assertEqual(lang_count, 0)
#
#
#class ModelMenuTest(TestCase):
#
#    def setUp(self):
#
#        self.lang = Language(language='Czech', country_code='CZ')
#        self.lang.save()
#        self.menu = Menu(lang=self.lang, menu_name='Menu')
#        self.menu.save()
#
#    def test_unicode(self):
#
#        self.assertEqual(self.menu.__unicode__(), self.menu.menu_name)
#
#
#class ModelMenuItemTest(TestCase):
#
#    def setUp(self):
#
#        self.lang = Language(language='English', country_code='EN')
#        self.lang.save()
#        self.lang2 = Language(language='Czech', country_code='CZ')
#        self.lang2.save()
#        self.menu = Menu(lang=self.lang, menu_name='Menu_english')
#        self.menu.save()
#        self.menu2 = Menu(lang=self.lang2, menu_name='Menu2_english')
#        self.menu2.save()
#
#        self.menuitem1 = MenuItem(menu=self.menu, menuitem_name='Menuitem1', url='a')
#        self.menuitem1.save()
#        self.menuitem2 = MenuItem(menu=self.menu, menuitem_name='Menuitem2', url='ag')
#        self.menuitem2.save()
#        self.menuitem3 = MenuItem(menu=self.menu, menuitem_name='Menuitem4', url='gg')
#        self.menuitem3.save()
#
#        self.page = Page(link=self.menuitem1, title='title', content='cac', active=True, index=True)
#        self.page.save()
#
#        self.menu2item1 = MenuItem(menu=self.menu2, menuitem_name='Menu2item1', url='m2i1')
#        self.menu2item1.save()
#        self.menu2item2 = MenuItem(menu=self.menu2, menuitem_name='Menu2item2', url='m2i2')
#        self.menu2item2.save()
#
#    def test_unicode(self):
#
#        self.assertEqual(self.menuitem1.__unicode__(), '%s@%s' % (self.menuitem1.menuitem_name, self.menuitem1.menu.menu_name))
#
#    def test_menuitems_positions(self):
#
#        self.assertEqual(self.menuitem1.position, 1)
#        self.assertEqual(self.menuitem2.position, 2)
#        self.assertEqual(self.menuitem3.position, 3)
#
#        self.assertEqual(self.menu2item1.position, 1)
#        self.assertEqual(self.menu2item2.position, 2)
#
#    def test_get_last_position(self):
#
#        self.assertEqual(self.menuitem1.get_last_position(), 3)
#        self.assertEqual(self.menu2item2.get_last_position(), 2)
#
#    def test_is_first(self):
#
#        self.assertTrue(self.menuitem1.is_first())
#        self.assertFalse(self.menuitem3.is_first())
#        self.assertTrue(self.menu2item1.is_first())
#        self.assertFalse(self.menu2item2.is_first())
#
#    def test_is_last(self):
#
#        self.assertTrue(self.menuitem3.is_last())
#        self.assertFalse(self.menuitem2.is_last())
#        self.assertTrue(self.menu2item2.is_last())
#        self.assertFalse(self.menu2item1.is_last())
#
#    def test_position_increase(self):
#
#        self.menuitem3.increase_position()
#
#        self.assertEqual(self.menuitem3.position, 3)
#
#        self.menuitem1.increase_position()
#
#        menuitem2_id = self.menuitem2.id
#        self.menuitem2 = MenuItem.objects.get(id=menuitem2_id)
#
#        self.assertEqual(self.menuitem1.position, 2)
#        self.assertEqual(self.menuitem2.position, 1)
#
#    def test_position_decrease(self):
#
#        self.menuitem1.decrease_position()
#        self.menuitem3.decrease_position()
#
#        menuitem2_id = self.menuitem2.id
#        self.menuitem2 = MenuItem.objects.get(id=menuitem2_id)
#
#        self.assertEqual(self.menuitem1.position, 1)
#        self.assertEqual(self.menuitem3.position, 2)
#        self.assertEqual(self.menuitem2.position, 3)
#
#    def test_is_current(self):
#
#        self.assertTrue(self.menuitem1.is_current('a'))
#        self.assertTrue(self.menuitem2.is_current('ag'))
#        self.assertFalse(self.menuitem1.is_current('not current'))
#        self.assertTrue(self.menuitem1.is_current(None))
#
#
#class ModelMetaSetTest(TestCase):
#
#    def setUp(self):
#
#        self.language = Language(language='English', country_code='EN')
#        self.language.save()
#        self.metaset = MetaSet(language=self.language, name='metaset')
#        self.metaset.save()
#
#    def test_unicode(self):
#
#        self.assertEqual(self.metaset.__unicode__(), self.metaset.name)
#
#
#class ModelMetaDataTest(TestCase):
#
#    def setUp(self):
#
#        self.language = Language(language='English', country_code='EN')
#        self.language.save()
#
#        self.metaset = MetaSet(language=self.language, name='metaset')
#        self.metaset.save()
#
#        self.metadata = MetaData(meta_set=self.metaset, name='keywords', content='klicova slova')
#        self.metadata.save()
#
#    def test_unicode(self):
#
#        self.assertEqual(self.metadata.__unicode__(), '%s@%s' % (self.metadata.name, self.metaset.name))
#
#
#class ModelPageTest(TestCase):
#
#    def setUp(self):
#
#        self.lang = Language(language='English', country_code='EN')
#        self.lang.save()
#        self.menu = Menu(lang=self.lang, menu_name='menu')
#        self.menu.save()
#        self.menuitem = MenuItem(menu=self.menu, menuitem_name='Some Page', url='somewhere')
#        self.menuitem.save()
#
#        self.page = Page(link=self.menuitem, title='Title', content='asdf', active=True, index=True)
#        self.page.save()
#
#    def test_unicode(self):
#
#        self.assertEqual(self.page.__unicode__(), self.page.title)
#
#    def test_have_posts(self):
#
#        self.assertFalse(self.page.have_posts())
#
#        self.post = Post(page=self.page, title='post', content='content')
#        self.post.save()
#
#        self.assertTrue(self.page.have_posts())
#
#
#class ModelPostTest(TestCase):
#
#    def setUp(self):
#
#        self.lang = Language(language='Czech', country_code='CZ')
#        self.lang.save()
#        self.menu = Menu(menu_name='menu1', lang=self.lang)
#        self.menu.save()
#        self.menuitem = MenuItem(menu=self.menu, menuitem_name='page1', url='homepage')
#        self.menuitem.save()
#        self.page = Page(link=self.menuitem, title='homepage', content='awesome homepage')
#        self.page.save()
#        self.post = Post(page=self.page, title='My first post', content='Nice text here', active=True)
#        self.post.save()
#
#        self.tz = get_current_timezone()
#
#    def test_unicode(self):
#
#        self.assertEqual(self.post.__unicode__(), self.post.title)
#
#    def test_post_is_visible(self):
#
#        self.assertTrue(self.post.is_visible(datetime(2012, 3, 5)))
#        self.assertTrue(self.post.is_visible(datetime(2014, 4, 5)))
#
#    def test_post_visible_from(self):
#
#        self.post.visible_from = make_aware(datetime(2013, 3, 17), self.tz)
#        self.post.save()
#
#        self.assertTrue(self.post.is_visible(datetime(2013, 3, 18)))
#        self.assertFalse(self.post.is_visible(datetime(203, 3, 16)))
#
#    def test_post_visible_to(self):
#
#        self.post.visible_to = make_aware(datetime(2013, 3, 17), self.tz)
#        self.post.save()
#
#        self.assertTrue(self.post.is_visible(datetime(2013, 3, 17)))
#        self.assertTrue(self.post.is_visible(datetime(2013, 3, 16)))
#        self.assertFalse(self.post.is_visible(datetime(2013, 3, 18)))
#
#    def test_post_visible_from_to(self):
#
#        self.post.visible_from = make_aware(datetime(2013, 3, 15), self.tz)
#        self.post.visible_to = make_aware(datetime(2013, 3, 20), self.tz)
#        self.post.save()
#
#        self.assertTrue(self.post.is_visible(datetime(2013, 3, 15)))
#        self.assertFalse(self.post.is_visible(datetime(2013, 3, 14)))
#        self.assertFalse(self.post.is_visible(datetime(2013, 3, 21)))
#        self.assertTrue(self.post.is_visible(datetime(2013, 3, 20)))
#
#
#class ModelCommentTest(TestCase):
#
#    'nothing to test here yet'
#
#    pass
#
#
#class ViewUrlTest(TestCase):
#
#    def test_404(self):
#
#        with self.assertRaises(Http404):
#
#            parse_url('endd/asFDSAf3~da3')
#
#    def test_language(self):
#
#        self.assertEqual(parse_url('en/'), {'page': None, 'post': None, 'page_num': None, 'country_code': 'en'})
#
#    def test_page(self):
#
#        self.assertEqual(parse_url('mypage'), {'page': 'mypage', 'post': None, 'page_num': None, 'country_code': None})
#
#    def test_post(self):
#
#        self.assertEqual(parse_url('/~mypost'), {'page': None, 'post': 'mypost', 'page_num': None, 'country_code': None})
#
#    def test_page_num(self):
#
#        self.assertEqual(parse_url('~0'), {'page': None, 'post': None, 'page_num': '0', 'country_code': None})
#
#    def test_all(self):
#
#        url = 'en/my-homepage~5/~post-on-5th-page-of-homepage'
#
#        result = {'country_code': 'en', 'page': 'my-homepage', 'page_num': '5', 'post': 'post-on-5th-page-of-homepage'}
#
#        self.assertEqual(parse_url(url), result)



# -*- encoding: utf-8 -*-

from django.core.cache import get_cache

from .models import Template


def get_template():
    """
    Returns current active template
    If there is none user-defined template, return 'default'

    @return (string, int)
    """

    try:

        cache = get_cache('default')

        if not cache.get('template'):

            template = Template.objects.get(active=True)
            template_name = template.template
            posts_count = template.posts_per_page

            current_template = (template_name, posts_count)
            cache.set('template', current_template, 30)

        else:

            current_template = cache.get('template')

    except Template.DoesNotExist:

        current_template = ('default', 10)

    return current_template



# -*- encoding: utf-8 -*-

from django.contrib import admin

import reversion

from ..common.admin_actions import activate, deactivate


class PageAdmin(reversion.VersionAdmin):

    fieldsets = (
        (None, {
            'fields': (('title', 'link'), 'content', 'active', 'index')
        }),
        ('Advanced options', {
            'classes': ('collapse',),
            'fields': ('metadata_set',)
        }),
    )

    list_display = ('title', 'link', 'active', 'metadata_set')
    actions = [activate, deactivate]

    class Media:

        js = [
            '/static/admin/js/ckeditor/ckeditor.js',
            '/static/admin/js/ckeditor/config.js',
            '/static/admin/js/ckeditor/start.js',
        ]

        css = {
            'ckeditor': ('/static/admin/js/ckeditor/contents.css',),
        }


class PostAdmin(reversion.VersionAdmin):

    fields = (('title', 'page'), 'content', 'active', 'comments', 'visible_from', 'visible_to', 'created')
    list_display = ('title', 'page', 'active')
    actions = [activate, deactivate]

    class Media:

        js = [
            '/static/admin/js/ckeditor/ckeditor.js',
            '/static/admin/js/ckeditor/config.js',
            '/static/admin/js/ckeditor/start.js',
        ]

        css = {
            'ckeditor': ('/static/admin/js/ckeditor/contents.css',),
        }



# -*- encoding: utf-8 -*-

import logging


class RequestLog(object):

    def process_request(self, request):

        log = logging.getLogger('requestlog')
        log.info('REQUEST: %s' % unicode(request).replace('\n', ' ').encode('utf-8'))



# -*- encoding: utf8 -*-

import datetime

from django.db import models

from django.utils.text import slugify
from django.utils.timezone import make_aware, get_current_timezone

from ..menu.models import MenuItem
from ..metadata.models import MetaSet


class Page(models.Model):
    '''
    Stores page data.
    Please be careful with comments, and don't put them on the Page and also.
    to all the Posts on that Page.
    Is related to :model: `django_pages.MenuItem`.
    '''

    link = models.OneToOneField(MenuItem)
    title = models.CharField('Page title', max_length=500)
    content = models.TextField('Page content', blank=True)

    metadata_set = models.ForeignKey(MetaSet, null=True, blank=True)

    active = models.BooleanField('Active', blank=True, default=True)
    index = models.BooleanField('Index page', blank=True)

    def __unicode__(self):

        return self.title

    def have_posts(self):
        """
        checks if Page has any Posts

        @return bool
        """

        if self.post_set.count():

            return True

        return False


class Post(models.Model):
    '''
    Stores post data.
    Posts are ment to be small paragraphs of text, sorted by date of creation.
    from newer (up) to older (down).
    Is related to :model: `django_pages.Page`.
    '''

    page = models.ForeignKey(Page)
    title = models.CharField('Post title', max_length=200, unique=True)
    content = models.TextField('Post content')
    comments = models.BooleanField('Enable comments', blank=True)

    active = models.BooleanField('Active', blank=True, default=True)

    visible_from = models.DateTimeField('Visible from', blank=True, null=True)
    visible_to = models.DateTimeField('Visible to', blank=True, null=True)

    created = models.DateTimeField('Created', blank=True, null=True)

    def __unicode__(self):

        return self.title

    def save(self, *args, **kwargs):

        if not self.created:

            self.created = make_aware(datetime.datetime.now(), get_current_timezone())

        super(Post, self).save(*args, **kwargs)

    def is_visible(self, now):
        """
        checks if the Post should be displayed or not

        @return bool
        """

        now = make_aware(now, get_current_timezone())

        if self.visible_from:

            if self.visible_from <= now:

                second_from = True

            else:

                second_from = False

        else:

            second_from = True

        if self.visible_to:

            if self.visible_to >= now:

                second_to = True

            else:

                second_to = False

        else:

            second_to = True

        if second_from and second_to:

            return True

        else:

            return False

    def get_url(self):
        """
        creates url for Post

        @return string
        """

        url_scheme = '/{country_code}/{link_url}/~{post_title}'

        title = slugify(self.title)

        result = url_scheme.format(country_code=self.page.link.lang.country_code, link_url=self.page.link.url, post_title=title)

        return result
